@@ -42,6 +42,8 @@ struct fio_client {
 	int skip_newline;
 	int is_sock;
 	int disk_stats_shown;
+	unsigned int jobs;
+	int error;
 
 	struct flist_head eta_list;
 	struct client_eta *eta_in_flight;
@@ -58,8 +60,9 @@ enum {
 	Client_created		= 0,
 	Client_connected	= 1,
 	Client_started		= 2,
-	Client_stopped		= 3,
-	Client_exited		= 4,
+	Client_running		= 3,
+	Client_stopped		= 4,
+	Client_exited		= 5,
 };
 
 static FLIST_HEAD(client_list);
@@ -769,6 +772,25 @@ static void handle_probe(struct fio_client *client, struct fio_net_cmd *cmd)
 		client->name = strdup((char *) probe->hostname);
 }
 
+static void handle_start(struct fio_client *client, struct fio_net_cmd *cmd)
+{
+	struct cmd_start_pdu *pdu = (struct cmd_start_pdu *) cmd->payload;
+
+	client->state = Client_started;
+	client->jobs = le32_to_cpu(pdu->jobs);
+}
+
+static void handle_stop(struct fio_client *client, struct fio_net_cmd *cmd)
+{
+	struct cmd_end_pdu *pdu = (struct cmd_end_pdu *) cmd->payload;
+
+	client->state = Client_stopped;
+	client->error = le32_to_cpu(pdu->error);
+
+	if (client->error)
+		log_info("client <%s>: exited with error %d\n", client->hostname, client->error);
+}
+
 static int handle_client(struct fio_client *client)
 {
 	struct fio_net_cmd *cmd;
@@ -824,12 +846,16 @@ static int handle_client(struct fio_client *client)
 		handle_probe(client, cmd);
 		free(cmd);
 		break;
+	case FIO_NET_CMD_RUN:
+		client->state = Client_running;
+		free(cmd);
+		break;
 	case FIO_NET_CMD_START:
-		client->state = Client_started;
+		handle_start(client, cmd);
 		free(cmd);
 		break;
 	case FIO_NET_CMD_STOP:
-		client->state = Client_stopped;
+		handle_stop(client, cmd);
 		free(cmd);
 		break;
 	default:
@@ -861,6 +887,8 @@ static void request_client_etas(void)
 			skipped++;
 			continue;
 		}
+		if (client->state != Client_running)
+			continue;
 
 		assert(!client->eta_in_flight);
 		flist_add_tail(&client->eta_list, &eta_list);
@@ -929,7 +957,7 @@ int fio_handle_clients(void)
 	struct fio_client *client;
 	struct flist_head *entry;
 	struct pollfd *pfds;
-	int i, ret = 0;
+	int i, ret = 0, retval = 0;
 
 	gettimeofday(&eta_tv, NULL);
 
@@ -986,10 +1014,12 @@ int fio_handle_clients(void)
 				log_info("client: host=%s disconnected\n",
 						client->hostname);
 				remove_client(client);
-			}
+				retval = 1;
+			} else if (client->error)
+				retval = 1;
 		}
 	}
 
 	free(pfds);
-	return 0;
+	return retval;
 }
