@@ -8,14 +8,38 @@
 #include <math.h>
 
 #include "fio.h"
+#include "smalloc.h"
+#include "diskutil.h"
 
 static int last_majdev, last_mindev;
+static struct disk_util *last_du;
 
-static struct flist_head disk_list = FLIST_HEAD_INIT(disk_list);
+FLIST_HEAD(disk_list);
+
+static struct disk_util *__init_per_file_disk_util(struct thread_data *td,
+		int majdev, int mindev, char *path);
+
+static void disk_util_free(struct disk_util *du)
+{
+	if (du == last_du)
+		last_du = NULL;
+
+	while (!flist_empty(&du->slaves)) {
+		struct disk_util *slave;
+
+		slave = flist_entry(du->slaves.next, struct disk_util, slavelist);
+		flist_del(&slave->slavelist);
+		slave->users--;
+	}
+	
+	fio_mutex_remove(du->lock);
+	sfree(du);
+}
 
 static int get_io_ticks(struct disk_util *du, struct disk_util_stat *dus)
 {
 	unsigned in_flight;
+	unsigned long long sectors[2];
 	char line[256];
 	FILE *f;
 	char *p;
@@ -36,13 +60,15 @@ static int get_io_ticks(struct disk_util *du, struct disk_util_stat *dus)
 	dprint(FD_DISKUTIL, "%s: %s", du->path, p);
 
 	ret = sscanf(p, "%u %u %llu %u %u %u %llu %u %u %u %u\n", &dus->ios[0],
-					&dus->merges[0], &dus->sectors[0],
+					&dus->merges[0], &sectors[0],
 					&dus->ticks[0], &dus->ios[1],
-					&dus->merges[1], &dus->sectors[1],
+					&dus->merges[1], &sectors[1],
 					&dus->ticks[1], &in_flight,
 					&dus->io_ticks, &dus->time_in_queue);
 	fclose(f);
 	dprint(FD_DISKUTIL, "%s: stat read ok? %d\n", du->path, ret == 1);
+	dus->sectors[0] = sectors[0];
+	dus->sectors[1] = sectors[1];
 	return ret != 11;
 }
 
@@ -51,6 +77,8 @@ static void update_io_tick_disk(struct disk_util *du)
 	struct disk_util_stat __dus, *dus, *ldus;
 	struct timeval t;
 
+	if (!du->users)
+		return;
 	if (get_io_ticks(du, &__dus))
 		return;
 
@@ -69,7 +97,7 @@ static void update_io_tick_disk(struct disk_util *du)
 	dus->time_in_queue += (__dus.time_in_queue - ldus->time_in_queue);
 
 	fio_gettime(&t, NULL);
-	du->msec += mtime_since(&du->time, &t);
+	dus->msec += mtime_since(&du->time, &t);
 	memcpy(&du->time, &t, sizeof(t));
 	memcpy(ldus, &__dus, sizeof(__dus));
 }
@@ -167,12 +195,8 @@ static int read_block_dev_entry(char *path, int *maj, int *min)
 	return 0;
 }
 
-
-static void __init_per_file_disk_util(struct thread_data *td, int majdev,
-		int mindev, char * path);
-
 static void find_add_disk_slaves(struct thread_data *td, char *path,
-		struct disk_util *masterdu)
+				 struct disk_util *masterdu)
 {
 	DIR *dirhandle = NULL;
 	struct dirent *dirent = NULL;
@@ -181,13 +205,14 @@ static void find_add_disk_slaves(struct thread_data *td, char *path,
 	int majdev, mindev;
 	ssize_t linklen;
 
-	sprintf(slavesdir, "%s/%s",path, "slaves");
+	sprintf(slavesdir, "%s/%s", path, "slaves");
 	dirhandle = opendir(slavesdir);
 	if (!dirhandle)
 		return;
 
 	while ((dirent = readdir(dirhandle)) != NULL) {
-		if (!strcmp(dirent->d_name, ".") || !strcmp(dirent->d_name, ".."))
+		if (!strcmp(dirent->d_name, ".") ||
+		    !strcmp(dirent->d_name, ".."))
 			continue;
 
 		sprintf(temppath, "%s/%s", slavesdir, dirent->d_name);
@@ -195,11 +220,12 @@ static void find_add_disk_slaves(struct thread_data *td, char *path,
 		 * are links to the real directories for the slave
 		 * devices?
 		 */
-		if ((linklen = readlink(temppath, slavepath, PATH_MAX-1)) < 0) {
+		linklen = readlink(temppath, slavepath, PATH_MAX - 0);
+		if (linklen  < 0) {
 			perror("readlink() for slave device.");
 			return;
 		}
-		slavepath[linklen]='\0';
+		slavepath[linklen] = '\0';
 
 		sprintf(temppath, "%s/%s/dev", slavesdir, slavepath);
 		if (read_block_dev_entry(temppath, &majdev, &mindev)) {
@@ -207,63 +233,70 @@ static void find_add_disk_slaves(struct thread_data *td, char *path,
 			return;
 		}
 
+		/*
+		 * See if this maj,min already exists
+		 */
+		slavedu = disk_util_exists(majdev, mindev);
+		if (slavedu)
+			continue;
+
 		sprintf(temppath, "%s/%s", slavesdir, slavepath);
 		__init_per_file_disk_util(td, majdev, mindev, temppath);
 		slavedu = disk_util_exists(majdev, mindev);
 
 		/* Should probably use an assert here. slavedu should
 		 * always be present at this point. */
-		if (slavedu)
+		if (slavedu) {
+			slavedu->users++;
 			flist_add_tail(&slavedu->slavelist, &masterdu->slaves);
+		}
 	}
 
 	closedir(dirhandle);
-	return;
 }
 
-
-
-static void disk_util_add(struct thread_data * td, int majdev, int mindev,
-		char *path)
+static struct disk_util *disk_util_add(struct thread_data *td, int majdev,
+				       int mindev, char *path)
 {
 	struct disk_util *du, *__du;
 	struct flist_head *entry;
 
 	dprint(FD_DISKUTIL, "add maj/min %d/%d: %s\n", majdev, mindev, path);
 
-	du = malloc(sizeof(*du));
+	du = smalloc(sizeof(*du));
 	memset(du, 0, sizeof(*du));
 	INIT_FLIST_HEAD(&du->list);
 	sprintf(du->path, "%s/stat", path);
-	du->name = strdup(basename(path));
+	strncpy((char *) du->dus.name, basename(path), FIO_DU_NAME_SZ);
 	du->sysfs_root = path;
 	du->major = majdev;
 	du->minor = mindev;
 	INIT_FLIST_HEAD(&du->slavelist);
 	INIT_FLIST_HEAD(&du->slaves);
+	du->lock = fio_mutex_init(1);
+	du->users = 0;
 
 	flist_for_each(entry, &disk_list) {
 		__du = flist_entry(entry, struct disk_util, list);
 
-		dprint(FD_DISKUTIL, "found %s in list\n", __du->name);
+		dprint(FD_DISKUTIL, "found %s in list\n", __du->dus.name);
 
-		if (!strcmp(du->name, __du->name)) {
-			free(du->name);
-			free(du);
-			return;
+		if (!strcmp((char *) du->dus.name, (char *) __du->dus.name)) {
+			disk_util_free(du);
+			return __du;
 		}
 	}
 
-	dprint(FD_DISKUTIL, "add %s to list\n", du->name);
+	dprint(FD_DISKUTIL, "add %s to list\n", du->dus.name);
 
 	fio_gettime(&du->time, NULL);
 	get_io_ticks(du, &du->last_dus);
 
 	flist_add_tail(&du->list, &disk_list);
 	find_add_disk_slaves(td, path, du);
+	return du;
 }
 
-
 static int check_dev_match(int majdev, int mindev, char *path)
 {
 	int major, minor;
@@ -271,9 +304,8 @@ static int check_dev_match(int majdev, int mindev, char *path)
 	if (read_block_dev_entry(path, &major, &minor))
 		return 1;
 
-	if (majdev == major && mindev == minor) {
+	if (majdev == major && mindev == minor)
 		return 0;
-	}
 
 	return 1;
 }
@@ -330,9 +362,9 @@ static int find_block_dir(int majdev, int mindev, char *path, int link_ok)
 	return found;
 }
 
-
-static void __init_per_file_disk_util(struct thread_data *td, int majdev,
-		int mindev, char * path)
+static struct disk_util *__init_per_file_disk_util(struct thread_data *td,
+						   int majdev, int mindev,
+						   char *path)
 {
 	struct stat st;
 	char tmp[PATH_MAX];
@@ -349,7 +381,7 @@ static void __init_per_file_disk_util(struct thread_data *td, int majdev,
 		sprintf(tmp, "%s/queue", p);
 		if (stat(tmp, &st)) {
 			log_err("unknown sysfs layout\n");
-			return;
+			return NULL;
 		}
 		strncpy(tmp, p, PATH_MAX - 1);
 		sprintf(path, "%s", tmp);
@@ -358,20 +390,19 @@ static void __init_per_file_disk_util(struct thread_data *td, int majdev,
 	if (td->o.ioscheduler && !td->sysfs_root)
 		td->sysfs_root = strdup(path);
 
-	disk_util_add(td, majdev, mindev, path);
+	return disk_util_add(td, majdev, mindev, path);
 }
 
-
-
-static void init_per_file_disk_util(struct thread_data *td, char * filename)
+static struct disk_util *init_per_file_disk_util(struct thread_data *td,
+						 char *filename)
 {
 
 	char foo[PATH_MAX];
 	struct disk_util *du;
 	int mindev, majdev;
 
-	if(get_device_numbers(filename, &majdev, &mindev))
-		return;
+	if (get_device_numbers(filename, &majdev, &mindev))
+		return NULL;
 
 	dprint(FD_DISKUTIL, "%s belongs to maj/min %d/%d\n", filename, majdev,
 			mindev);
@@ -381,7 +412,7 @@ static void init_per_file_disk_util(struct thread_data *td, char * filename)
 		if (td->o.ioscheduler && !td->sysfs_root)
 			td->sysfs_root = strdup(du->sysfs_root);
 
-		return;
+		return du;
 	}
 
 	/*
@@ -391,20 +422,20 @@ static void init_per_file_disk_util(struct thread_data *td, char * filename)
 	 * everything again.
 	 */
 	if (mindev == last_mindev && majdev == last_majdev)
-		return;
+		return last_du;
 
 	last_mindev = mindev;
 	last_majdev = majdev;
 
 	sprintf(foo, "/sys/block");
 	if (!find_block_dir(majdev, mindev, foo, 1))
-		return;
-
-	__init_per_file_disk_util(td, majdev, mindev, foo);
+		return NULL;
 
+	return __init_per_file_disk_util(td, majdev, mindev, foo);
 }
 
-static void __init_disk_util(struct thread_data *td, struct fio_file *f)
+static struct disk_util *__init_disk_util(struct thread_data *td,
+					  struct fio_file *f)
 {
 	return init_per_file_disk_util(td, f->file_name);
 }
@@ -419,114 +450,139 @@ void init_disk_util(struct thread_data *td)
 		return;
 
 	for_each_file(td, f, i)
-		__init_disk_util(td, f);
+		f->du = __init_disk_util(td, f);
+}
+
+static void show_agg_stats(struct disk_util_agg *agg, int terse)
+{
+	if (!agg->slavecount)
+		return;
+
+	if (!terse) {
+		log_info(", aggrios=%u/%u, aggrmerge=%u/%u, aggrticks=%u/%u,"
+				" aggrin_queue=%u, aggrutil=%3.2f%%",
+				agg->ios[0] / agg->slavecount,
+				agg->ios[1] / agg->slavecount,
+				agg->merges[0] / agg->slavecount,
+				agg->merges[1] / agg->slavecount,
+				agg->ticks[0] / agg->slavecount,
+				agg->ticks[1] / agg->slavecount,
+				agg->time_in_queue / agg->slavecount,
+				agg->max_util.u.f);
+	} else {
+		log_info("slaves;%u;%u;%u;%u;%u;%u;%u;%3.2f%%",
+				agg->ios[0] / agg->slavecount,
+				agg->ios[1] / agg->slavecount,
+				agg->merges[0] / agg->slavecount,
+				agg->merges[1] / agg->slavecount,
+				agg->ticks[0] / agg->slavecount,
+				agg->ticks[1] / agg->slavecount,
+				agg->time_in_queue / agg->slavecount,
+				agg->max_util.u.f);
+	}
 }
 
 static void aggregate_slaves_stats(struct disk_util *masterdu)
 {
+	struct disk_util_agg *agg = &masterdu->agg;
 	struct disk_util_stat *dus;
 	struct flist_head *entry;
 	struct disk_util *slavedu;
-	double util, max_util = 0;
-	int slavecount = 0;
-
-	unsigned merges[2] = { 0, };
-	unsigned ticks[2] = { 0, };
-	unsigned time_in_queue = { 0, };
-	unsigned long long sectors[2] = { 0, };
-	unsigned ios[2] = { 0, };
+	double util;
 
 	flist_for_each(entry, &masterdu->slaves) {
 		slavedu = flist_entry(entry, struct disk_util, slavelist);
 		dus = &slavedu->dus;
-		ios[0] += dus->ios[0];
-		ios[1] += dus->ios[1];
-		merges[0] += dus->merges[0];
-		merges[1] += dus->merges[1];
-		sectors[0] += dus->sectors[0];
-		sectors[1] += dus->sectors[1];
-		ticks[0] += dus->ticks[0];
-		ticks[1] += dus->ticks[1];
-		time_in_queue += dus->time_in_queue;
-		++slavecount;
+		agg->ios[0] += dus->ios[0];
+		agg->ios[1] += dus->ios[1];
+		agg->merges[0] += dus->merges[0];
+		agg->merges[1] += dus->merges[1];
+		agg->sectors[0] += dus->sectors[0];
+		agg->sectors[1] += dus->sectors[1];
+		agg->ticks[0] += dus->ticks[0];
+		agg->ticks[1] += dus->ticks[1];
+		agg->time_in_queue += dus->time_in_queue;
+		agg->slavecount++;
 
-		util = (double) (100 * dus->io_ticks / (double) slavedu->msec);
+		util = (double) (100 * dus->io_ticks / (double) slavedu->dus.msec);
 		/* System utilization is the utilization of the
 		 * component with the highest utilization.
 		 */
-		if (util > max_util)
-			max_util = util;
+		if (util > agg->max_util.u.f)
+			agg->max_util.u.f = util;
 
 	}
 
-	if (max_util > 100.0)
-		max_util = 100.0;
-
-	log_info(", aggrios=%u/%u, aggrmerge=%u/%u, aggrticks=%u/%u,"
-			" aggrin_queue=%u, aggrutil=%3.2f%%",
-			ios[0]/slavecount, ios[1]/slavecount,
-			merges[0]/slavecount, merges[1]/slavecount,
-			ticks[0]/slavecount, ticks[1]/slavecount,
-			time_in_queue/slavecount, max_util);
-
+	if (agg->max_util.u.f > 100.0)
+		agg->max_util.u.f = 100.0;
 }
 
-
-
-void show_disk_util(void)
+void free_disk_util(void)
 {
-	struct disk_util_stat *dus;
-	struct flist_head *entry, *next;
 	struct disk_util *du;
+
+	while (!flist_empty(&disk_list)) {
+		du = flist_entry(disk_list.next, struct disk_util, list);
+		flist_del(&du->list);
+		disk_util_free(du);
+	}
+
+	last_majdev = last_mindev = -1;
+}
+
+void print_disk_util(struct disk_util_stat *dus, struct disk_util_agg *agg,
+		     int terse)
+{
 	double util;
 
+	util = (double) 100 * dus->io_ticks / (double) dus->msec;
+	if (util > 100.0)
+		util = 100.0;
+
+	if (agg->slavecount)
+		log_info("  ");
+
+	if (!terse) {
+		log_info("  %s: ios=%u/%u, merge=%u/%u, ticks=%u/%u, "
+			 "in_queue=%u, util=%3.2f%%", dus->name,
+					dus->ios[0], dus->ios[1],
+					dus->merges[0], dus->merges[1],
+					dus->ticks[0], dus->ticks[1],
+					dus->time_in_queue, util);
+	} else {
+		log_info(";%s;%u;%u;%u;%u;%lu;%lu;%u;%u;%u;%u;%3.2f%%",
+					dus->name, dus->ios[0], dus->ios[1],
+					dus->merges[0], dus->merges[1],
+					dus->ticks[0], dus->ticks[1],
+					dus->time_in_queue, util);
+	}
+
+	/*
+	 * If the device has slaves, aggregate the stats for
+	 * those slave devices also.
+	 */
+	if (agg->slavecount)
+		show_agg_stats(agg, terse);
+
+	if (!terse)
+		log_info("\n");
+}
+
+void show_disk_util(int terse)
+{
+	struct flist_head *entry;
+	struct disk_util *du;
+
 	if (flist_empty(&disk_list))
 		return;
 
-	log_info("\nDisk stats (read/write):\n");
+	if (!terse)
+		log_info("\nDisk stats (read/write):\n");
 
 	flist_for_each(entry, &disk_list) {
 		du = flist_entry(entry, struct disk_util, list);
-		dus = &du->dus;
 
-		util = (double) 100 * du->dus.io_ticks / (double) du->msec;
-		if (util > 100.0)
-			util = 100.0;
-
-		/* If this node is the slave of a master device, as
-		 * happens in case of software RAIDs, inward-indent
-		 * this stats line to reflect a master-slave
-		 * relationship. Because the master device gets added
-		 * before the slave devices, we can safely assume that
-		 * the master's stats line has been displayed in a
-		 * previous iteration of this loop.
-		 */
-		if(!flist_empty(&du->slavelist))
-			log_info("  ");
-
-		log_info("  %s: ios=%u/%u, merge=%u/%u, ticks=%u/%u, "
-			 "in_queue=%u, util=%3.2f%%", du->name,
-						dus->ios[0], dus->ios[1],
-						dus->merges[0], dus->merges[1],
-						dus->ticks[0], dus->ticks[1],
-						dus->time_in_queue, util);
-
-		/* If the device has slaves, aggregate the stats for
-		 * those slave devices also.
-		 */
-		if(!flist_empty(&du->slaves))
-			aggregate_slaves_stats(du);
-
-		log_info("\n");
-	}
-
-	/*
-	 * now free the list
-	 */
-	flist_for_each_safe(entry, next, &disk_list) {
-		flist_del(entry);
-		du = flist_entry(entry, struct disk_util, list);
-		free(du->name);
-		free(du);
+		aggregate_slaves_stats(du);
+		print_disk_util(&du->dus, &du->agg, terse);
 	}
 }
