@@ -262,7 +262,7 @@ void show_group_stats(struct group_run_stats *rs)
 	const char *ddir_str[] = { "   READ", "  WRITE" };
 	int i;
 
-	log_info("Run status group %d (all jobs):\n", rs->groupid);
+	log_info("\nRun status group %d (all jobs):\n", rs->groupid);
 
 	for (i = 0; i <= DDIR_WRITE; i++) {
 		const int i2p = is_power_of_2(rs->kb_base);
@@ -441,26 +441,39 @@ static void show_ddir_status(struct group_run_stats *rs, struct thread_stat *ts,
 	}
 	if (calc_lat(&ts->bw_stat[ddir], &min, &max, &mean, &dev)) {
 		double p_of_agg;
+		const char *bw_str = "KB";
 
 		p_of_agg = mean * 100 / (double) rs->agg[ddir];
-		log_info("     bw (KB/s) : min=%5lu, max=%5lu, per=%3.2f%%,"
-			 " avg=%5.02f, stdev=%5.02f\n", min, max, p_of_agg,
-							mean, dev);
+		if (p_of_agg > 100.0)
+			p_of_agg = 100.0;
+
+		if (mean > 999999.9) {
+			min /= 1000.0;
+			max /= 1000.0;
+			mean /= 1000.0;
+			dev /= 1000.0;
+			bw_str = "MB";
+		}
+
+		log_info("    bw (%s/s)  : min=%5lu, max=%5lu, per=%3.2f%%,"
+			 " avg=%5.02f, stdev=%5.02f\n", bw_str, min, max,
+							p_of_agg, mean, dev);
 	}
 }
 
-static void show_lat(double *io_u_lat, int nr, const char **ranges,
-		     const char *msg)
+static int show_lat(double *io_u_lat, int nr, const char **ranges,
+		    const char *msg)
 {
-	int new_line = 1, i, line = 0;
+	int new_line = 1, i, line = 0, shown = 0;
 
 	for (i = 0; i < nr; i++) {
 		if (io_u_lat[i] <= 0.0)
 			continue;
+		shown = 1;
 		if (new_line) {
 			if (line)
 				log_info("\n");
-			log_info("     lat (%s): ", msg);
+			log_info("    lat (%s) : ", msg);
 			new_line = 0;
 			line = 0;
 		}
@@ -471,6 +484,11 @@ static void show_lat(double *io_u_lat, int nr, const char **ranges,
 		if (line == 5)
 			new_line = 1;
 	}
+
+	if (shown)
+		log_info("\n");
+
+	return shown;
 }
 
 static void show_lat_u(double *io_u_lat_u)
@@ -493,9 +511,7 @@ static void show_lat_m(double *io_u_lat_m)
 static void show_latencies(double *io_u_lat_u, double *io_u_lat_m)
 {
 	show_lat_u(io_u_lat_u);
-	log_info("\n");
 	show_lat_m(io_u_lat_m);
-	log_info("\n");
 }
 
 void show_thread_status(struct thread_stat *ts, struct group_run_stats *rs)
@@ -520,7 +536,7 @@ void show_thread_status(struct thread_stat *ts, struct group_run_stats *rs)
 					ts->error, ts->verror, (int) ts->pid);
 	}
 
-	if (ts->description)
+	if (strlen(ts->description))
 		log_info("  Description  : [%s]\n", ts->description);
 
 	if (ts->io_bytes[DDIR_READ])
@@ -528,6 +544,10 @@ void show_thread_status(struct thread_stat *ts, struct group_run_stats *rs)
 	if (ts->io_bytes[DDIR_WRITE])
 		show_ddir_status(rs, ts, DDIR_WRITE);
 
+	stat_calc_lat_u(ts, io_u_lat_u);
+	stat_calc_lat_m(ts, io_u_lat_m);
+	show_latencies(io_u_lat_u, io_u_lat_m);
+
 	runtime = ts->total_run_time;
 	if (runtime) {
 		double runt = (double) runtime;
@@ -561,14 +581,12 @@ void show_thread_status(struct thread_stat *ts, struct group_run_stats *rs)
 					io_u_dist[1], io_u_dist[2],
 					io_u_dist[3], io_u_dist[4],
 					io_u_dist[5], io_u_dist[6]);
-	log_info("     issued r/w/d: total=%lu/%lu/%lu, short=%lu/%lu/%lu\n",
+	log_info("     issued    : total=r=%lu/w=%lu/d=%lu,"
+				 " short=r=%lu/w=%lu/d=%lu\n",
 					ts->total_io_u[0], ts->total_io_u[1],
 					ts->total_io_u[2],
 					ts->short_io_u[0], ts->short_io_u[1],
 					ts->short_io_u[2]);
-	stat_calc_lat_u(ts, io_u_lat_u);
-	stat_calc_lat_m(ts, io_u_lat_m);
-	show_latencies(io_u_lat_u, io_u_lat_m);
 	if (ts->continue_on_error) {
 		log_info("     errors    : total=%lu, first_error=%d/<%s>\n",
 					ts->total_err_count,
@@ -654,8 +672,8 @@ static void show_thread_status_terse(struct thread_stat *ts,
 	int i;
 
 	/* General Info */
-	log_info("%s;%s;%d;%d", FIO_TERSE_VERSION, ts->name, ts->groupid,
-				ts->error);
+	log_info("%s;%s;%s;%d;%d", FIO_TERSE_VERSION, fio_version_string,
+					ts->name, ts->groupid, ts->error);
 	/* Log Read Status */
 	show_ddir_status_terse(ts, rs, 0);
 	/* Log Write Status */
@@ -994,25 +1012,23 @@ void show_run_stats(void)
 			show_thread_status(ts, rs);
 	}
 
-	if (!terse_output) {
-		for (i = 0; i < groupid + 1; i++) {
-			rs = &runstats[i];
-
-			rs->groupid = i;
-			if (is_backend)
-				fio_server_send_gs(rs);
-			else
-				show_group_stats(rs);
-		}
+	for (i = 0; i < groupid + 1; i++) {
+		rs = &runstats[i];
 
+		rs->groupid = i;
 		if (is_backend)
-			fio_server_send_du();
-		else
-			show_disk_util(0);
-
-		free_disk_util();
+			fio_server_send_gs(rs);
+		else if (!terse_output)
+			show_group_stats(rs);
 	}
 
+	if (is_backend)
+		fio_server_send_du();
+	else if (!terse_output)
+		show_disk_util(0);
+
+	free_disk_util();
+
 	free(runstats);
 	free(threadstats);
 }
